package net.sf.saxon.instruct;
import net.sf.saxon.Controller;
import net.sf.saxon.ParameterSet;

import javax.xml.transform.TransformerException;

/**
* The compiled form of an xsl:attribute-set element in the stylesheet.
*/

// Note, there is no run-time check for circularity. This is checked at compile time.

public class AttributeSet {

    private AttributeSet[] useAttributeSets;     // the value of the use-attribute-sets attribute, as supplied
    private Block body;
    private boolean needsStackFrame;

    public AttributeSet() {}

    public void initialize( AttributeSet[] useAttributeSets,
                            Block body,
                            boolean needsStackFrame ) {
        this.useAttributeSets = useAttributeSets;
        this.body = body;
        this.needsStackFrame = needsStackFrame;
    }

    public void expand(Controller controller) throws TransformerException {
        // apply the content of any attribute sets mentioned in use-attribute-sets

        if (useAttributeSets != null) {
            AttributeSet.expand(useAttributeSets, controller);
        }

        // Note, we can ignore the result of processLeavingTail() since an attribute set can't contain a tail call

        if (needsStackFrame) {
            Bindery bindery = controller.getBindery();
            bindery.openStackFrame(ParameterSet.EMPTY_PARAMETER_SET, null);
            body.processLeavingTail(controller.newXPathContext());
            bindery.closeStackFrame();
        } else {
            body.processLeavingTail(controller.newXPathContext());
        }
    }

    /**
     * Expand an array of attribute sets
     * @param asets the attribute sets to be expanded
     * @param controller the run-time context to use
     * @throws TransformerException
     */

    protected static void expand(AttributeSet[] asets, Controller controller) throws TransformerException {
        for (int i=0; i<asets.length; i++) {
            asets[i].expand(controller);
        }
    }
}

//
// The contents of this file are subject to the Mozilla Public License Version 1.0 (the "License");
// you may not use this file except in compliance with the License. You may obtain a copy of the
// License at http://www.mozilla.org/MPL/
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied.
// See the License for the specific language governing rights and limitations under the License.
//
// The Original Code is: all this file.
//
// The Initial Developer of the Original Code is Michael H. Kay.
//
// Portions created by (your name) are Copyright (C) (your legal entity). All Rights Reserved.
//
// Contributor(s): none.
//
