package net.sf.saxon.type;

import java.util.Iterator;

/**
 * This interface represents a collection of facets. It contains methods that
 * allow the facets for a simple type to be retrieved individually or collectively,
 * with or without the facets inherited from the base type. The concrete implementation
 * is in the schema-aware product.
 */
public interface FacetCollection {

    /**
     * Returns the first facet associated with the given name, defined either on this type
     * or on a type from which it is derived
     * @return the first facet associated with the given name
    **/

    Facet getFacet(String name);

    /**
     * Return all the facets having the given name
     * @param name the required facet name, for example "totalDigits" or "maxInclusive"
     * @return an iterator over all the facets with the given name, including those defined
     * on a type from which this is derived by restriction or by union. The iterator contains
     * objects of class {@link Facet}.
     */

    Iterator getFacets(String name);

    /**
     * Returns an Iterator over all the Facets (including inherited facets)
     *  for this type.
     * @return an Iterator over all the Facets for this type. The iterator contains
     * objects of class {@link Facet}.
    */

    Iterator getFacets();

}
