package net.sf.saxon.type;

import java.io.Serializable;

/**
 * This object represents a global validation context. In XQuery, this is used
 * whenever the validation context is global. In XSLT, it is used for all validation,
 * except where a named type is specified.
 */

public class GlobalValidationContext implements ValidationContext, Serializable {

    // This class has a singleton instance

    private static GlobalValidationContext theInstance =
            new GlobalValidationContext();

    /**
     * Get the singleton instance of this class
     * @return the singleton instance
     */

    public static GlobalValidationContext getInstance() {
        return theInstance;
    }

    /**
     * Determine whether this is a void validation context (a context in which nothing will validate)
     * @return false: this validation context is not void
     */

    public boolean isVoidValidationContext() {
        return false;
    }

    /**
     * The constructor is private
     */

    private GlobalValidationContext() {}

}
